package anywhersoftware.b4j.objects;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.*;

import java.util.concurrent.ExecutionException;


import org.java_websocket.client.WebSocketClient;
import org.java_websocket.drafts.Draft;
import org.java_websocket.drafts.Draft_6455;
import org.java_websocket.handshake.ServerHandshake;

import org.java_websocket.extensions.IExtension;
import org.java_websocket.protocols.IProtocol;
import org.java_websocket.protocols.Protocol;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

/**
 * Implementation of a WebSocket client.
 */
@ShortName("WebSocketClient")
@Events(values={"Open (ServerHandshake handshake)", "Close (code As Int, reason As String, remote As Boolean)", "Message (Message As String)", "Error"})
@Version(1.6f)
@DependsOn(values={"Java-WebSocket"})
public class WebSocketClientWrapper {
	private String eventName;
	@Hide
	public WebSocketClient wsc;

	/**
	 * Initializes the object and sets the subs that will handle the events.
	 */	
	public void Initialize(final BA ba, String EventName, final String Url, final Map<String,String> httpHeaders, final int timeout) throws URISyntaxException {
		eventName = EventName.toLowerCase(BA.cul);
		URI echoUri = new URI(Url);

//		How to change this in B4J?
//		Draft_6455 protocolDraft = new Draft_6455(Collections.<IExtension>emptyList(), Collections.<IProtocol>singletonList(new Protocol("MyName")));
		Draft_6455 protocolDraft = new Draft_6455();

		wsc = new WebSocketClient(echoUri, protocolDraft, httpHeaders, timeout) {
			@Override
			public void onOpen(ServerHandshake handshake) {
				ba.raiseEventFromDifferentThread(WebSocketClientWrapper.this, null, 0, eventName + "_open", false, new Object[] {handshake});
			}
			@Override
			public void onMessage(String message) {
				ba.raiseEventFromDifferentThread(WebSocketClientWrapper.this, null, 0, eventName + "_message", false, new Object[] {message});
			}
			@Override
			public void onError(Exception ex) {
//				ex.printStackTrace();
				ba.setLastException(ex);
				ba.raiseEventFromDifferentThread(WebSocketClientWrapper.this, null, 0, eventName + "_error", false, new Object[] {});
			}
			@Override
			public void onClose(int code, String reason, boolean remote) {
				ba.raiseEventFromDifferentThread(WebSocketClientWrapper.this, null, 0, eventName + "_close", false, new Object[] {code, reason, remote});
			}
		};
	}
	public void Connect() {
		wsc.connect();
	}
	public void Reconnect() {
		wsc.reconnect();
	}
	public void Close() {
		wsc.close();
	}
	public void SendText(String Text) throws IOException, InterruptedException, ExecutionException {
		wsc.send(Text);
	}
	public void SendByte(byte[] data) throws IOException, InterruptedException, ExecutionException {
		wsc.send(data);
	}
	public Boolean isConnected() {
		return wsc.isOpen();
	}
	public String getURI() {
		String URI;
		URI = wsc.getURI().toString();
		return URI;
	}
}
